@@ -1,7 +1,7 @@
 /*
  * Copyright 2008 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanmacho.c,v 1.5 2008/10/19 18:11:59 grobian Exp $
+ * $Header: /var/cvsroot/gentoo-projects/pax-utils/scanmacho.c,v 1.17 2010/01/15 11:06:33 vapier Exp $
  *
  * based on scanelf by:
  * Copyright 2003-2007 Ned Ludd        - <solar@gentoo.org>
@@ -10,7 +10,7 @@
  *                2008 Fabian Groffen  - <grobian@gentoo.org>
  */
 
-static const char *rcsid = "$Id: scanmacho.c,v 1.5 2008/10/19 18:11:59 grobian Exp $";
+static const char *rcsid = "$Id: scanmacho.c,v 1.17 2010/01/15 11:06:33 vapier Exp $";
 const char * const argv0 = "scanmacho";
 
 #include "paxinc.h"
@@ -22,7 +22,7 @@ static int scanmacho_fatobj(fatobj *fobj);
 static int scanmacho_file(const char *filename, const struct stat *st_cache);
 static int scanmacho_from_file(const char *filename);
 static int scanmacho_dir(const char *path);
-static void scanelf_envpath(void);
+static void scanmacho_envpath(void);
 static void usage(int status);
 static int parseargs(int argc, char *argv[]);
 
@@ -50,27 +50,8 @@ static char *out_format = NULL;
 static char *search_path = NULL;
 static char g_match = 0;
 
-int match_bits = 0;
-unsigned int match_perms = 0;
-caddr_t ldcache = 0;
-size_t ldcache_size = 0;
-unsigned long setpax = 0UL;
-
-int has_objdump = 0;
-
-static char *getstr_perms(const char *fname);
-static char *getstr_perms(const char *fname)
-{
-	struct stat st;
-	static char buf[8];
-
-	if ((stat(fname, &st)) == (-1))
-		return (char *) "";
-
-	snprintf(buf, sizeof(buf), "%o", st.st_mode);
-
-	return (char *) buf + 2;
-}
+static int match_bits = 0;
+static unsigned int match_perms = 0;
 
 static const char *macho_file_needed_lib(
 		fatobj *fobj,
@@ -80,28 +61,20 @@ static const char *macho_file_needed_lib(
 		char **ret,
 		size_t *ret_len
 ) {
-	char *needed;
 	loadcmd *lcmd;
-	struct dylib_command *dlcmd;
-	uint32_t lc_load_dylib = LC_LOAD_DYLIB;
+	uint32_t lc_load_dylib;
 
 	if ((op == 0 && !show_needed) || (op == 1 && !find_lib))
 		return NULL;
 
 	lcmd = firstloadcmd(fobj);
-
-	if (fobj->swapped)
-		lc_load_dylib = bswap_32(lc_load_dylib);
+	lc_load_dylib = MGET(fobj->swapped, LC_LOAD_DYLIB);
 
 	do {
 		if (lcmd->lcmd->cmd == lc_load_dylib) {
-			dlcmd = (struct dylib_command*)lcmd->data;
-			if (fobj->swapped) {
-				needed = (char *)(lcmd->data +
-						bswap_32(dlcmd->dylib.name.offset));
-			} else {
-				needed = (char *)(lcmd->data + dlcmd->dylib.name.offset);
-			}
+			struct dylib_command *dlcmd = lcmd->data;
+			char *needed;
+			needed = lcmd->data + MGET(fobj->swapped, dlcmd->dylib.name.offset);
 			if (op == 0) {
 				if (!be_wewy_wewy_quiet) {
 					if (*found_needed)
@@ -130,71 +103,55 @@ static const char *macho_file_needed_lib(
 static char *macho_file_interp(fatobj *fobj, char *found_interp)
 {
 	loadcmd *lcmd;
-	uint32_t lc_load_dylinker = LC_LOAD_DYLINKER;
+	uint32_t lc_load_dylinker;
 
 	if (!show_interp)
 		return NULL;
 
 	lcmd = firstloadcmd(fobj);
-
-	if (fobj->swapped)
-		lc_load_dylinker = bswap_32(lc_load_dylinker);
+	lc_load_dylinker = MGET(fobj->swapped, LC_LOAD_DYLINKER);
 
 	do {
 		if (lcmd->lcmd->cmd == lc_load_dylinker) {
-			struct dylinker_command *dlcmd =
-				(struct dylinker_command*)lcmd->data;
+			struct dylinker_command *dlcmd = lcmd->data;
 			char *dylinker;
-			if (fobj->swapped) {
-				dylinker = (char *)(lcmd->data +
-						bswap_32(dlcmd->name.offset));
-			} else {
-				dylinker = (char *)(lcmd->data + dlcmd->name.offset);
-			}
+			dylinker = lcmd->data + MGET(fobj->swapped, dlcmd->name.offset);
 			*found_interp = 1;
 			free(lcmd);
 			return (be_wewy_wewy_quiet ? NULL : dylinker);
 		}
 	} while (nextloadcmd(lcmd));
-	
+
 	return NULL;
 }
 
 static char *macho_file_soname(fatobj *fobj, char *found_soname)
 {
 	loadcmd *lcmd;
-	char *soname;
-	uint32_t lc_id_dylib = LC_ID_DYLIB;
+	uint32_t lc_id_dylib;
 
 	if (!show_soname)
 		return NULL;
 
 	lcmd = firstloadcmd(fobj);
-
-	if (fobj->swapped)
-		lc_id_dylib = bswap_32(lc_id_dylib);
+	lc_id_dylib = MGET(fobj->swapped, LC_ID_DYLIB);
 
 	do {
 		if (lcmd->lcmd->cmd == lc_id_dylib) {
-			struct dylib_command *dlcmd = (struct dylib_command*)lcmd->data;
-			if (fobj->swapped) {
-				soname = (char *)(lcmd->data +
-						bswap_32(dlcmd->dylib.name.offset));
-			} else {
-				soname = (char *)(lcmd->data + dlcmd->dylib.name.offset);
-			}
+			struct dylib_command *dlcmd = lcmd->data;
+			char *soname;
+			soname = lcmd->data + MGET(fobj->swapped, dlcmd->dylib.name.offset);
 			*found_soname = 1;
 			free(lcmd);
 			return (be_wewy_wewy_quiet ? NULL : soname);
 		}
 	} while (nextloadcmd(lcmd));
-	
+
 	return NULL;
 }
 
-
 /* scan a macho file and show all the fun stuff */
-#define prints(str) write(fileno(stdout), str, strlen(str))
+#define prints(str) ({ ssize_t ret = write(fileno(stdout), str, strlen(str)); ret; })
 static int scanmacho_fatobj(fatobj *fobj)
 {
 	unsigned long i;
@@ -296,7 +253,7 @@ static int scanmacho_fatobj(fatobj *fobj)
 		case 'o': out = get_machomhtype(fobj); break;
 		case 'M': out = get_machocputype(fobj); break;
 		case 'D': out = get_machoendian(fobj); break;
-		case 'O': out = getstr_perms(fobj->filename); break;
+		case 'O': out = strfileperms(fobj->filename); break;
 		case 'n':
 		case 'N': out = macho_file_needed_lib(fobj, &found_needed, &found_lib, (out_format[i]=='N'), &out_buffer, &out_len); break;
 		case 'i': out = macho_file_interp(fobj, &found_interp); break;
@@ -396,7 +353,7 @@ static int scanmacho_archive(const char *filename, int fd, size_t len)
 	if (ar == NULL)
 		return 1;
 
-	ar_buffer = (char*)mmap(0, len, PROT_READ, MAP_PRIVATE, fd, 0);
+	ar_buffer = mmap(0, len, PROT_READ, MAP_PRIVATE, fd, 0);
 	while ((m = ar_next(ar)) != NULL) {
 		fobj = readmacho_buffer(m->name, ar_buffer + lseek(fd, 0, SEEK_CUR), m->size);
 		if (fobj) {
@@ -465,9 +422,8 @@ static int scanmacho_dir(const char *path)
 	}
 
 	/* ok, if it isn't a directory, assume we can open it */
-	if (!S_ISDIR(st_top.st_mode)) {
+	if (!S_ISDIR(st_top.st_mode))
 		return scanmacho_file(path, &st_top);
-	}
 
 	/* now scan the dir looking for fun stuff */
 	if ((dir = opendir(path)) == NULL) {
@@ -523,9 +479,8 @@ static int scanmacho_from_file(const char *filename)
 	return ret;
 }
 
-
 /* scan env PATH for paths */
-static void scanelf_envpath(void)
+static void scanmacho_envpath(void)
 {
 	char *path, *p;
 
@@ -579,7 +534,7 @@ static const char *opts_help[] = {
 	"Scan directories recursively",
 	"Don't recursively cross mount points",
 	"Don't scan symlinks",
-	"Scan archives (.a files)",
+	"Scan archives (.a files)\n",
 	"Print LC_LOAD_DYLIB information (ELF: NEEDED)",
 	"Print LC_LOAD_DYLINKER information (ELF: INTERP)",
 	"Print flags from mach_header (ELF: BIND)",
@@ -592,7 +547,7 @@ static const char *opts_help[] = {
 	"Print Endianness",
 	"Print only Mach-O files matching octal permissions",
 	"Print Mach-O file size",
-	"Print all scanned info (-F\"%o %O %D %b %F\")\n",
+	"Print all useful/simple info\n",
 	"Only output 'bad' things",
 	"Be verbose (can be specified more than once)",
 	"Use specified format for output",
@@ -652,7 +607,7 @@ static int parseargs(int argc, char *argv[])
 			match_bits = atoi(optarg);
 			break;
 		case 'O':
-			if (sscanf(optarg, "%o", &match_perms) == (-1))
+			if (sscanf(optarg, "%o", &match_perms) == -1)
 				match_bits = 0;
 			break;
 		case 'o': {
@@ -747,7 +702,7 @@ static int parseargs(int argc, char *argv[])
 
 	/* now lets actually do the scanning */
 	if (scan_envpath)
-		scanelf_envpath();
+		scanmacho_envpath();
 	if (!from_file && optind == argc && ttyname(0) == NULL && !scan_envpath)
 		from_file = "-";
 	if (from_file) {
